package mobicomp.net;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

import mobicomp.interfaces.Message;
import mobicomp.interfaces.SHSocket;


/**
 * TODO
 * 
 * @author Andreas Wetzel <andi@spamato.net>
 */
public class SHSocketImpl implements SHSocket
{
    public static final int MAX_PACKET_SIZE = 0xFFFF;
    
    private int localAddress;

    private InetAddress group;
    private MulticastSocket socket;

    public SHSocketImpl( int addr) throws IOException {
        // setup network properties
        group = InetAddress.getByName( GROUP_ADDRESS);
        socket = new MulticastSocket( COMMUNICATION_PORT);
        socket.joinGroup( group);

        setOwnAddress( addr);
    }

    /**
     * @see mobicomp.interfaces.SHSocket#setOwnAddress(int)
     */
    public void setOwnAddress( int addr) {
        localAddress = addr;
    }

    /**
     * @see mobicomp.interfaces.SHSocket#getOwnAddress()
     */
    public int getOwnAddress() {
        return localAddress;
    }

    /**
     * @see mobicomp.interfaces.SHSocket#send(mobicomp.interfaces.Message)
     */
    public void send( Message p) throws IOException {
        // we use a ByteArrayOutputStream to convert the Message to an array of bytes
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream( bos);

        dos.writeShort( p.getSender());
        dos.writeShort( p.getReceiver());
        dos.write( p.getData());

        dos.close();
        byte[] buffer = bos.toByteArray();

        // construct and send the DatagramPacket
        DatagramPacket packet = new DatagramPacket( buffer, buffer.length, group, COMMUNICATION_PORT);
        socket.send( packet);
    }

    /**
     * @see mobicomp.interfaces.SHSocket#receive()
     */
    public Message receive() throws IOException {
        byte[] buffer = new byte[MAX_PACKET_SIZE];
        DatagramPacket packet = new DatagramPacket( buffer, buffer.length);

        Message message = new SHMessageImpl();
        do {
            socket.receive( packet);

            // we use a ByteArrayInputStream to easily parse the byte array.
            ByteArrayInputStream bis = new ByteArrayInputStream( packet.getData());
            DataInputStream dis = new DataInputStream( bis);

            message.setSender( dis.readUnsignedShort());
            message.setReceiver( dis.readUnsignedShort());
            byte[] data = new byte[packet.getLength()-4]; // size of data is full packet size minus header size
            dis.read( data);
            message.setData( data);
            
        // loop as long as the read message is not for 'us'
        } while ( (message.getReceiver() != localAddress) && (message.getReceiver() != BROADCAST_ADDRESS) );
        
        return message;
    }

}